/***************************************************************************
  filejump.cpp  -
  --------------------------------------------------------
  Maintainer: Stefan Gehn <sgehn@gmx.net>

  Copyright (c) 2003      Stefan Gehn <sgehn@gmx.net>

 ***************************************************************************/

#include "filejump.h"
#include "playlist.h"
#include "view.h"

#include <qlayout.h>
#include <qvbox.h>
#include <qlabel.h>
#include <qheader.h>
#include <qscrollbar.h>
#include <qtimer.h>
#include <qpushbutton.h>

#include <klineedit.h>
#include <klocale.h>
#include <kmessagebox.h>
//#include <kdebug.h>

FileJumpDialog::FileJumpDialog(QWidget *parent)
	: KDialogBase(parent, 0, false /*non-modal needs fix in noatun*/,
		i18n("Jump To File"), Close|User1, User1, false, i18n("&Jump"))
{
//	kdDebug(66666) << k_funcinfo << "Called." << endl;
	QWidget *mainWidget = makeMainWidget();
	mainWidget->setMinimumWidth(320);

	QVBoxLayout *layout = new QVBoxLayout(mainWidget, 0, KDialog::spacingHint(), "layout");
	QHBoxLayout *top = new QHBoxLayout(layout, KDialog::spacingHint(), "top");

	QLabel *lblSearch = new QLabel(i18n("Search:"), mainWidget, "lblSearch");
	top->addWidget(lblSearch);
	mText = new KLineEdit(mainWidget, "mText");
	top->addWidget(mText);

	mResultList = new KListView(mainWidget, "mResultList");
	mResultList->addColumn(QString::fromLatin1("Title"));
	mResultList->setFullWidth(TRUE);
	mResultList->header()->hide();
	mResultList->setFocusPolicy(QWidget::NoFocus);
	mResultList->setSorting(-1);

	layout->addWidget(mResultList);

	connect(mText, SIGNAL(textChanged(const QString &)),
		SLOT(textChanged(const QString &)));

	connect(this, SIGNAL(user1Clicked()), this, SLOT(slotJumpClicked()));
	connect(mResultList, SIGNAL(executed(QListViewItem*)), this, SLOT(slotJumpClicked()));

	mTimer = new QTimer(this, "FileJumpDialog::mTimer");
	connect(mTimer, SIGNAL(timeout()), this, SLOT(slotInputDelay()));
}

FileJumpDialog::~FileJumpDialog()
{
//	kdDebug(66666) << k_funcinfo << "Called." << endl;
}

void FileJumpDialog::show()
{
//	kdDebug(66666) << k_funcinfo << "Called." << endl;

	refillList(mText->text());
	mText->setFocus();
	// used in case we get non-modal in the future
	if (isVisible())
		raise();
	else
		KDialogBase::show();
}


void FileJumpDialog::textChanged(const QString &)
{
	mTimer->start(400, TRUE);
}


// TODO: refactor, used in both find and jump
static bool testWord(PlaylistItem &item, const QString &finder)
{
	//PlaylistItemData *item=static_cast<SafeListViewItem*>(i);
	if (item.title().find(finder, 0, false) >= 0)
		return true;
	if (item.file().find(finder, 0, false) >= 0)
		return true;
	if (item.url().path().find(finder.local8Bit(), 0, false) >= 0)
		return true;
/*
	if (item.lengthString().find(finder, 0, false) >=0)
		return true;
	if (item.mimetype().find(finder.local8Bit(), 0, false) >=0)
		return true;
*/
	return false;
}

void FileJumpDialog::refillList(const QString &searchString)
{
//	kdDebug(66666) << k_funcinfo << "Called, string=" << searchString << endl;

	KListViewItem *lItem = 0L;

	setUpdatesEnabled(FALSE);
	mResultList->clear();
	ll.clear();

	PlaylistItem item = FPL->getFirst();
	do
	{
//		if(item.title().contains(searchString, false))
		if(testWord(item, searchString))
		{
			lItem = new KListViewItem(mResultList, item.title());
			/*
			// Fix for old noatun bug?
			PlaylistItem FPLItem = item;
			ll.insert(lItem, FPLItem);
			*/
			ll.insert(lItem, item);
		}
	} while(item = FPL->getAfter(item));

	mResultList->setCurrentItem(mResultList->firstChild());
	mResultList->setSelected(mResultList->firstChild(), true);
	setUpdatesEnabled(TRUE);
}

void FileJumpDialog::slotInputDelay()
{
	refillList(mText->text());
}

void FileJumpDialog::slotJumpClicked()
{
	if (!mResultList->selectedItem())
		return;

//	kdDebug(66666) << k_funcinfo << "Called." << endl;

	if (mResultList->selectedItem() && ll.contains(mResultList->selectedItem()))
	{
		const PlaylistItem item = ll[mResultList->selectedItem()];
		if(!item.isNull())
			emit doJump(item);
		accept();
	}
}

void FileJumpDialog::keyPressEvent(QKeyEvent *e)
{
	QListViewItem *item = mResultList->currentItem();

	switch(e->key())
	{
		case Qt::Key_Up:
			item = item->itemAbove();
			break;

		case Qt::Key_Down:
			item = item->itemBelow();
			break;

		case Qt::Key_PageUp:
			for(int i = 0; i <
				(mResultList->verticalScrollBar()->pageStep()/item->height()); i++ )
			{
				if(!item->itemAbove())
					break;
				item = item->itemAbove();
			}
			break;

		case Qt::Key_PageDown:
			for(int i = 0; i <
				(mResultList->verticalScrollBar()->pageStep()/item->height()); i++ )
			{
				if(!item->itemBelow())
					break;
				item = item->itemBelow();
			}
			break;

		case Qt::Key_Escape:
			slotClose();
			break;

		case Qt::Key_Enter:
		case Qt::Key_Return:
			emit user1Clicked();
			break;

		default:
			e->ignore();
			return;
	}

	if (item)
	{
		if (item != mResultList->currentItem())
		{
			mResultList->ensureItemVisible(item);
			mResultList->setSelected(item, true);
			mResultList->setCurrentItem(item);
		}
	}
	e->accept();
}
#include "filejump.moc"
