/***************************************************************************
  find.cpp  -
  --------------------------------------------------------
  Maintainer: Stefan Gehn <sgehn@gmx.net>

  Copyright (c) 2000-2001 Charles Samuels <charles@kde.org>
                2000-2001 Neil Stevens <neil@qualityassistant.com>
                2003      Stefan Gehn <sgehn@gmx.net>

 ***************************************************************************/

#include "find.h"

#include <qlayout.h>
#include <qpushbutton.h>
#include <qcheckbox.h>

#include <kcombobox.h>
#include <klocale.h>
#include <kdebug.h>

Finder::Finder(QWidget *parent)
	: KDialogBase(parent, "Finder", false, i18n("Find"), Close | User1, User1, false, i18n("&Find"))
{
//	kdDebug(66666) << k_funcinfo << "Called." << endl;
	QWidget *mainWidget = makeMainWidget(); //new QWidget(this);
	mainWidget->setMinimumWidth(320);

	QGridLayout *layout = new QGridLayout(mainWidget, 1, 1, 0, KDialog::spacingHint(), "Finder::layout");

	mText = new KHistoryCombo(mainWidget, "Finder::mText");
	mText->setMaxCount(10);
	mText->setFocus();

	mRegexp = new QCheckBox(i18n("&Regular expression"), mainWidget);
	mBackwards = new QCheckBox(i18n("Find &backwards"), mainWidget);

	layout->addMultiCellWidget(mText, 0, 0, 0, 1);
	layout->addWidget(mRegexp, 1, 0);
	layout->addWidget(mBackwards, 1, 1);

	connect(this, SIGNAL(user1Clicked()), SLOT(clicked()));
	connect(mText, SIGNAL(activated(int)), SLOT(clicked()));
	connect(mText, SIGNAL(textChanged(const QString &)), SLOT(textChanged(const QString &)));

	enableButton(User1, false);
}

Finder::~Finder()
{
//	kdDebug(66666) << k_funcinfo << "Called." << endl;
}

void Finder::textChanged(const QString &text)
{
	enableButton(User1, !text.isEmpty());
}

bool Finder::regexp() const
{
	return mRegexp->isChecked();
}

bool Finder::isForward() const
{
	return !mBackwards->isChecked();
}

void Finder::clicked()
{
	mText->addToHistory(mText->currentText());
	emit search(this);
}

QString Finder::string() const
{
	return mText->currentText();
}

#include "find.moc"
