/***************************************************************************
  flexprefs.cpp  -  Preferences-Dialog for FlexPlaylist
  --------------------------------------------------------
  Maintainer: Stefan Gehn <sgehn@gmx.net>
  Copyright (c) 2003      Stefan Gehn <sgehn@gmx.net>

 ***************************************************************************/

// system includes
#include "flexprefs.h"

#include <qcheckbox.h>
#include <qcolor.h>
#include <qlabel.h>
#include <qlayout.h>
#include <qspinbox.h>

#include <klocale.h>
#include <kglobalsettings.h>
#include <kdebug.h>
#include <kconfig.h>
#include <kfontcombo.h>
#include <kcolorcombo.h>

FlexPrefs::FlexPrefs(QObject* parent)
	: CModule(i18n("Flex Playlist"), i18n("Flexible Playlist"), "view_text", parent)
{
	cfg = KGlobal::config();
	QVBoxLayout *vbox = new QVBoxLayout(this);
	vbox->setAutoAdd(true);
	vbox->setSpacing(0);
	vbox->setMargin(0);
	mMainWidget = new FlexPrefsWidget(this, "mMainWidget");
	reopen();
}

FlexPrefs::~FlexPrefs()
{

}

void FlexPrefs::reopen() // reload config and set stuff in dialog
{
	cfg->setGroup("flexplaylist");
	mModified = cfg->readBoolEntry("modified", false);
	mFile = cfg->readEntry("file", QString::null);
	mSaved = cfg->readNumEntry("current", 0);

	QFont *tmpFnt = new QFont(KGlobalSettings::generalFont());
	mFont = cfg->readFontEntry("font",tmpFnt);
	delete tmpFnt;

	mShowHeader = cfg->readBoolEntry("Show ListView Header", true);
	mShowStatusButtons = cfg->readBoolEntry("Show StatusBar Buttons", false);

	QColor *tmpColor = new QColor(0,0,0);
	mSelBgColor = cfg->readColorEntry("Selected Background Color", tmpColor);

	tmpColor->setRgb(255,255,255);
	mSelTxtColor = cfg->readColorEntry("Selected Text Color", tmpColor);
	delete tmpColor;

	mSelUseBold = cfg->readBoolEntry("Selected Text Bold", true);
	mShowIcons = cfg->readBoolEntry("Show Icons", true);
	mRememberGeom = cfg->readBoolEntry("Remember Window Geometry", true);

	mMainWidget->mFontCombo->setCurrentFont(mFont.family());
	mMainWidget->mFontSizeSpin->setValue(mFont.pointSize());
	mMainWidget->mShowHeader->setChecked(mShowHeader);
	mMainWidget->cmbSelTxtColor->setColor(mSelTxtColor);
	mMainWidget->cmbSelBgColor->setColor(mSelBgColor);
	mMainWidget->chkSelBold->setChecked(mSelUseBold);
	mMainWidget->chkShowIcons->setChecked(mShowIcons);
	mMainWidget->chkRememberGeom->setChecked(mRememberGeom);
	mMainWidget->mShowStatusButtons->setChecked(mShowStatusButtons);
}

void FlexPrefs::save()
{
	mFont.setFamily(mMainWidget->mFontCombo->currentFont());
	mFont.setPointSize(mMainWidget->mFontSizeSpin->value());
	mShowHeader		= mMainWidget->mShowHeader->isChecked();
	mSelUseBold		= mMainWidget->chkSelBold->isChecked();
	mSelTxtColor	= mMainWidget->cmbSelTxtColor->color();
	mSelBgColor		= mMainWidget->cmbSelBgColor->color();
	mShowIcons		= mMainWidget->chkShowIcons->isChecked();
	mRememberGeom	= mMainWidget->chkRememberGeom->isChecked();
	mShowStatusButtons = mMainWidget->mShowStatusButtons->isChecked();

	cfg->setGroup("flexplaylist");
	cfg->writeEntry("modified", mModified);
	cfg->writeEntry("file", mFile);
	cfg->writeEntry("current", mSaved);
	cfg->writeEntry("font", mFont);
	cfg->writeEntry("Show ListView Header", mShowHeader);
	cfg->writeEntry("Show StatusBar Buttons", mShowStatusButtons);
	cfg->writeEntry("Selected Background Color", mSelBgColor);
	cfg->writeEntry("Selected Text Color", mSelTxtColor);
	cfg->writeEntry("Selected Text Bold", mSelUseBold);
	cfg->writeEntry("Show Icons", mShowIcons);
	cfg->writeEntry("Remember Window Geometry", mRememberGeom);

	cfg->sync();
	emit configChanged();
}

void FlexPrefs::setSaved(int s)
{
	mSaved = s;
}

void FlexPrefs::setFile(const QString &file)
{
	mFile = file;
}

void FlexPrefs::setModified(bool mod)
{
	mModified = mod;
}

#include "flexprefs.moc"
