#ifndef PLAYLIST_H
#define PLAYLIST_H

#include <noatun/playlist.h>
#include <noatun/plugin.h>
#include "flexprefs.h"

class SafeListViewItem;
class View;
class FPListView;
class QListViewItem;

class FlexPlaylist : public Playlist, public Plugin, PlaylistNotifier
{
	Q_OBJECT
	friend class SafeListViewItem;
	friend class FPListView;

	public:
		FlexPlaylist();
		~FlexPlaylist();

		/*
		 * go to the front of the playlist
		 */
		virtual void reset();

		virtual void clear();
		virtual void addFile(const KURL&, bool play=false);

		/*
		 * Cycle forward playlistitems and return new current item
		 */
		virtual PlaylistItem next();
		PlaylistItem next(bool play);

		/*
		 * return current PlaylistItem that should be playing now
		 */
		virtual PlaylistItem current();

		/*
		 * Cycle through backwards
		 */
		virtual PlaylistItem previous();

		virtual PlaylistItem getFirst() const;
		virtual PlaylistItem getAfter(const PlaylistItem &item) const;

		virtual bool listVisible() const;
		virtual void init();

		/*
		 * Called when a playlistitem changed length
		 * Needed to compute length of whole playlist
		 */
		void lengthChanged(QString &, QString &);

		virtual Playlist *playlist() { return this; }
		FlexPrefs *prefs() const { return mPrefs; }

		static FlexPlaylist *FPL() { return Self; }
		const unsigned long fullLength() { return mFullLength; }
		inline bool exiting() const { return mExiting; }

		// from playlistnotifier
		virtual void removed(PlaylistItem &);

		void updating(bool b);
		bool updating();

	public slots:
		virtual void showList();
		virtual void hideList();
		virtual void remove(const PlaylistItem&);
		virtual void sort();

		void setCurrent(const PlaylistItem &);
		void setCurrent(const PlaylistItem &, bool emitC);
		void listItemSelected(QListViewItem*);
		void randomize();

	signals:
		void play(PlaylistItem*);

	private:
		unsigned long stringToLen(QString &);

	private:
		PlaylistItem currentItem, randomPrevious;
		bool mExiting; // HACK HACK HACK HACK!!!
		bool mUpdating; // flag set in case of huge list updates (adding or removal of many playlistitems)
		View *view;
		static FlexPlaylist *Self;
		FlexPrefs *mPrefs;
		unsigned long mFullLength;
};
#endif
